from gocept.munin.client import SimpleMultiGraph, main
from munin.zope.memory import vmkeys
from os.path import isdir, basename, abspath, normpath, join, exists
from os import environ, getcwd, symlink
from inspect import isclass
from socket import gethostname
from sys import argv


class zodbactivity(SimpleMultiGraph):
    keys = ['total_load_count', 'total_store_count', 'total_connections']
    names = ['Total_objects_loaded', 'Total_objects_stored',
             'Total_connections']
    title = 'ZODB Activity'
    category = 'Zope'


class zopecache(SimpleMultiGraph):
    keys = ['total_objs', 'total_objs_memory', 'target_number']
    names = ['Total_objects_in_database', 'Total_objects_in_all_caches',
             'Target_number_to_cache']
    title = 'Zope cache parameters'
    category = 'Zope'


class zopethreads(SimpleMultiGraph):
    keys = ['total_threads', 'free_threads']
    names = ['Total_threads', 'Free_threads']
    title = 'Z2Server threads'
    category = 'Zope'


keys = vmkeys()
class zopememory(SimpleMultiGraph):
    keys = keys
    names = keys
    title = 'Zope memory usage'
    category = 'Zope'


def install(script, cmd, path, prefix=None, suffix=None):
    """ set up plugin symlinks using the given prexix/suffix or the
        current hostname and directory """
    assert isdir(path), 'please specify an existing directory'
    if prefix is None:
        prefix = gethostname()
    if suffix is None:
        suffix = basename(getcwd())
    source = abspath(script)
    for name, value in globals().items():
        if isclass(value) and value.__module__ == __name__:
            plugin = '_'.join((prefix, name, suffix))
            target = normpath(join(path, plugin))
            if not exists(target):
                symlink(source, target)
                print 'installed symlink %s' % target
            else:
                print 'skipped existing %s' % target


def run(ip_address='localhost', http_address=8080, port_base=0, user=None):
    if 3 <= len(argv) <= 5 and argv[1] == 'install':
        return install(*argv)
    if not 'URL' in environ:
        port = int(http_address) + int(port_base)
        host = '%s:%d' % (ip_address, port)
        environ['URL'] = 'http://%s/@@munin.zope.plugins/%%s' % host
    if not 'AUTH' in environ and user is not None:
        environ['AUTH'] = user
    main()



from Products.Five.browser import BrowserView
from ZServer.PubCore.ZRendezvous import ZRendevous
from logging import getLogger
from munin.zope.memory import vmstats
from time import time
import sys
if sys.version_info < (2, 5):
    import threadframe
    thread = threadframe.dict
else:
    thread = sys._current_frames

log = getLogger('munin.zope').info

def timer(fn):
    def decorator(*args, **kw):
        start = time()
        value = fn(*args, **kw)
        elapsed = time() - start
        if elapsed > 0.1:   # only log when execution took more than 100ms
            log('calling %s took %.3fs', fn.__name__, elapsed)
        return value
    decorator.__doc__ = fn.__doc__
    decorator.__name__ = fn.__name__
    return decorator


class Munin(BrowserView):

    @timer
    def zopecache(self):
        """ zodb cache statistics """
        result = []
        db = self.context.unrestrictedTraverse('/Control_Panel/Database/main')
        result.append('total_objs:%.1f' % db.database_size())
        result.append('total_objs_memory:%.1f' % db.cache_length())
        result.append('target_number:%.1f' %
            (len(db.cache_detail_length()) * db.cache_size()))
        return '\n'.join(result)

    @timer
    def zodbactivity(self):
        """ zodb activity statistics """
        result = []
        now = time()
        start = now - 300   # munin's polls every 5 minutes (*60 seconds)
        end = now
        db = self.context.unrestrictedTraverse('/Control_Panel/Database/main')
        params = dict(chart_start=start, chart_end=end)
        chart = db.getActivityChartData(200, params)
        result.append('total_load_count:%.1f' % chart['total_load_count'])
        result.append('total_store_count:%.1f' % chart['total_store_count'])
        result.append('total_connections:%.1f' % chart['total_connections'])
        return '\n'.join(result)

    @timer
    def zopethreads(self):
        """ zope thread statistics """
        result = []
        frames = thread()
        total_threads = len(frames.values())
        free_threads = 0
        for frame in frames.values():
            _self = frame.f_locals.get('self')
            if hasattr(_self, '__module__') and \
                    _self.__module__ == ZRendevous.__module__:
                free_threads += 1
        result.append('total_threads:%.1f' % total_threads)
        result.append('free_threads:%.1f' % free_threads)
        return '\n'.join(result)

    @timer
    def zopememory(self):
        """ zope memory usage statistics """
        result = ['%s:%s' % item for item in vmstats()]
        return '\n'.join(result)



from os import getpid
from re import compile


def readfile(name):
    try:
        lines = open(name).readlines()
    except IOError:
        lines = ['VmSize: 0 kB']
    return lines


def vmstats(pid=getpid()):
    vm = compile(r'^(Vm[^:]*):\s*(\d+) kB$')
    for line in readfile('/proc/%s/status' % pid):
        match = vm.match(line)
        if match:
            field, value = match.groups()
            yield field, int(value) * 1024


def vmkeys():
    return [ key for key, value in vmstats() ]



from unittest import TestSuite
from zope.testing import doctest
from AccessControl.SecurityManagement import newSecurityManager
from Testing.ZopeTestCase import installPackage, utils
from Testing.ZopeTestCase import FunctionalTestCase, FunctionalDocFileSuite
from Products.Five import zcml, fiveconfigure
from Products.Five.testbrowser import Browser

optionflags = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)


class TestLayer:
    """ layer for integration tests """

    @classmethod
    def setUp(cls):
        # load zcml & install package
        fiveconfigure.debug_mode = True
        import munin.zope
        zcml.load_site()
        zcml.load_config('configure.zcml', munin.zope)
        fiveconfigure.debug_mode = False
        installPackage('munin.zope', quiet=True)

    @classmethod
    def tearDown(cls):
        pass


class TestCase(FunctionalTestCase):
    """ base class for functional tests;  please note that this test case
        creates another user on the application root, which is why it needs
        a special version of `setRoles` """

    layer = TestLayer

    def afterSetUp(self):
        uf = self.app.acl_users
        uf._doAddUser('manager', 's3kr3t', ['Member'], [])
        user = uf.getUserById('manager').__of__(uf)
        newSecurityManager(None, user)

    def setRoles(self, roles):
        uf = self.app.acl_users
        uf.userFolderEditUser('manager', None, utils.makelist(roles), [])

    def getBrowser(self, loggedIn=True):
        """ instantiate and return a testbrowser for convenience """
        browser = Browser()
        if loggedIn:
            browser.addHeader('Authorization',
                'Basic %s:%s' % ('manager', 's3kr3t'))
        return browser


def test_suite():
    return TestSuite([
        FunctionalDocFileSuite(
           'browser.txt', package='munin.zope',
           test_class=TestCase, optionflags=optionflags),
    ])



from setuptools import setup, find_packages
from os.path import join
import sys

version = '1.2'
readme = open("README.txt").read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = ['setuptools', 'gocept.munin']
if sys.version_info < (2, 5):
    install_requires.append('threadframe')

setup(name = 'munin.zope',
      version = version,
      description = 'Munin plugins for Zope/Plone.',
      long_description = readme[readme.find('\n\n'):] + '\n' + history,
      classifiers = [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Framework :: Plone',
        'Framework :: ZODB',
        'Framework :: Zope2',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Topic :: System :: Networking :: Monitoring',
        'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      keywords = 'plone zope munin monitoring',
      author = 'Andrew Mleczko',
      author_email = 'andrew.mleczko@redturtle.it',
      url = 'http://plone.org/products/munin.zope',
      license = 'GPL',
      packages = find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages = ['munin'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      install_requires = install_requires,
      entry_points = """
          [console_scripts]
          munin = munin.zope.plugins:run
      """,
)
